//
//  FBFAnimation.m
//  Created by Onur Ersel 15.02.2012.
//  Copyright (c) 2012 Onur Ersel. All rights reserved.
//

#import "FBFAnimation.h"

@implementation FBFAnimation
@synthesize animationArray, fps, isLooping, currentAnimationFrame;


#pragma mark - DEALLOC

-(void)dealloc
{
    if(displayLink != nil)
        [self stopAnimating];
    
    [animationArray release];
    [animationFramesArray release];
    [super dealloc];
}


#pragma mark - INIT

- (id)init {
    self = [super init];
    if (self) {
        isLooping = NO;
        fps = 30;
        
        self.contentMode = UIViewContentModeTopLeft;
        
        endSelector = NULL;
        endDelegate = nil;
        displayLink = nil;
        isAnimatingOverride = NO;
    }
    return self;
}

- (id)initWithSequenceName:(NSString*)constantNameValue   imageCount:(unsigned int)imageCountValue   numberStartsWith:(unsigned int)startValue   suffix:(NSString*)suffixValue   {
    self = [self init];
    if (self) {
        animationFramesArray = [[NSMutableArray alloc] init];
        NSMutableArray *tmpArray = [NSMutableArray array];
        for (int i = startValue; i < imageCountValue + startValue; i++) {
            NSString *name = [NSString stringWithFormat:@"%@%i", constantNameValue, i];
            NSString *path = [[NSBundle mainBundle] pathForResource:name ofType:suffixValue];
            [tmpArray addObject:path];
            [animationFramesArray addObject:[NSNumber numberWithInt:i-startValue]];
        }
        self.animationArray = [NSArray arrayWithArray:tmpArray];
        
        self.currentAnimationFrame = 0;
        [self setDisplayFrame:currentAnimationFrame];
    }
    return self;
}

- (id)initWithNameArray:(NSArray*)pNamesArray   suffix:(NSString*)pSuffix {
    self = [self init];
    if (self) {
        animationFramesArray = [[NSMutableArray alloc] init];
        NSMutableArray *tmpArray = [NSMutableArray array];
        for (int i = 0; i < [pNamesArray count]; i++) {
            NSString *path = [[NSBundle mainBundle] pathForResource:[pNamesArray objectAtIndex:i] ofType:pSuffix];
            [tmpArray addObject:path];
            [animationFramesArray addObject:[NSNumber numberWithInt:i]];
        }
        self.animationArray = [NSArray arrayWithArray:tmpArray];
        
        self.currentAnimationFrame = 0;
        [self setDisplayFrame:currentAnimationFrame];
    }
    return self;
}


#pragma mark - ANIMATION

-(void)startAnimating
{
    [self startAnimatingWithAnimationFramesArray:animationFramesArray andFPS:fps isLooping:isLooping startFrame:0];
    isAnimatingOverride = YES;
}

-(void)stopAnimating
{
    [displayLink invalidate];
    displayLink = nil;
    isAnimatingOverride = NO;
}

-(void)resumeAnimating
{
    if(displayLink != nil)
        [self stopAnimating];
    
    [self startAnimatingWithAnimationFramesArray:animationFramesArray andFPS:fps isLooping:isLooping startFrame:currentAnimationFrame];
}

-(void)startAnimatingWithAnimationFramesArray:(NSArray*)animationFramesArrayValue  andFPS:(unsigned int)fpsValue  isLooping:(BOOL)isLoopingValue   startFrame:(unsigned int)startFrameValue
{
    if(animationFramesArray != animationFramesArrayValue)
    {
        [animationFramesArray release];
        animationFramesArray = [[NSMutableArray arrayWithArray:animationFramesArrayValue] retain];
    }
    
    isLooping = isLoopingValue;
    fps = fpsValue;
    
    displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(update:)];
    [displayLink setFrameInterval:1];
    [displayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    
    startTime = CFAbsoluteTimeGetCurrent();
    currentAnimationFrame = startFrameValue;
    startFrame = startFrameValue;
}

-(void)startAnimatingWithFPS:(unsigned int)fpsValue
{
    [self startAnimatingWithAnimationFramesArray:animationFramesArray andFPS:fpsValue isLooping:isLooping startFrame:0];
}

-(BOOL)isAnimating
{
    return isAnimatingOverride;
}


#pragma mark - DELEGATE
-(void)setEndDelegate:(id)delegate  andSelector:(SEL)selector
{
    endSelector = selector;
    endDelegate = delegate;
}


#pragma mark - UPDATE

-(void)update:(CADisplayLink *)sender;
{
    float targetFrame = floor((CFAbsoluteTimeGetCurrent() - startTime) / (1.0/fps)) + startFrame;
    if(self.currentAnimationFrame != targetFrame)
        self.currentAnimationFrame = targetFrame;
    else
        return;
        
    if(self.currentAnimationFrame >= [animationFramesArray count])
    {
        if(self.isLooping == YES)
        {
            self.currentAnimationFrame = 0;
            startTime = CFAbsoluteTimeGetCurrent();
        }
        else
        {
            [self stopAnimating];
            if(endDelegate != nil  &&  endSelector != NULL)
                [endDelegate performSelector:endSelector withObject:self];
            return;
        }
    }
    
    [self setDisplayFrame:currentAnimationFrame];
}


#pragma mark - DISPLAY FRAME
-(void)setDisplayFrame:(unsigned int)displayFrameValue
{
    UIImage *img = [[UIImage alloc] initWithContentsOfFile:[self.animationArray objectAtIndex:[[animationFramesArray objectAtIndex:displayFrameValue] intValue]]];
    self.image = img;
    [self.image release];
}

@end
